@@ -9,9 +9,9 @@ Tk::PopList - Popping a selection list relative to a widget
 use strict;
 use warnings;
 use vars qw($VERSION);
-$VERSION = '0.01';
+$VERSION = '0.02';
 
-use base qw(Tk::Derived Tk::Toplevel);
+use base qw(Tk::Derived Tk::Poplevel);
 
 use Tk;
 use Tie::Watch;
@@ -29,6 +29,8 @@ Construct Tk::Widget 'PopList';
 
 =head1 DESCRIPTION
 
+Inherits L<Tk::Poplevel>
+
 This widget pops a listbox relative to the widget specified in the B<-widget> option.
 It aligns its size and position to the widget.
 
@@ -62,14 +64,6 @@ Callback, called when a list item is selected.
 
 List of possible values.
 
-=item B<-widget>
-
-Mandatory!
-
-Reference to the widget the list should pop relative to.
-
-Only available at create time.
-
 =back
 
 =head1 METHODS
@@ -84,20 +78,12 @@ sub Populate {
 	my $motionselect = delete $args->{'-motionselect'};
 	$motionselect = 1 unless defined $motionselect;
 
-	my $widget = delete $args->{'-widget'};
-	die 'You need to set the -widget option' unless defined $widget;
-
 	$self->SUPER::Populate($args);
 
 	$self->{FE} = undef;
 	$self->{LIST} = [];
-	$self->{POPDIRECTION} = '';
 	$self->{VALUES} = [];
-	$self->{WIDGET} = $widget;
 	
-	$self->overrideredirect(1);
-	$self->withdraw;
-
 	my $listbox = $self->Scrolled('Listbox',
 		-borderwidth => 1,
 		-relief => 'sunken',
@@ -105,11 +91,10 @@ sub Populate {
 		-listvariable => $self->{LIST},
 	)->pack(-fill => 'both');
 	$self->Advertise('Listbox', $listbox);
+	$listbox->bind('<Escape>', [$self, 'popDown']);
 	$listbox->bind('<Return>', [$self, 'Select']);
 	$listbox->bind('<ButtonRelease-1>', [$self, 'Select', Ev('x'), Ev('y')]);
-	$listbox->bind('<Escape>', [$self, 'popDown']);
 	$self->bind('<Motion>', [$self, 'MotionSelect', Ev('x'), Ev('y')]) if $motionselect;
-	$self->parent->bind('<Button-1>', [$self, 'popDown']);
 
 	$self->ConfigSpecs(
 		-filter => ['PASSIVE', undef, undef, 0],
@@ -119,33 +104,16 @@ sub Populate {
 	);
 }
 
-sub ConfigureSizeAndPos {
+sub calculateHeight {
 	my $self = shift;
 	my $list = $self->{LIST};
 	my $lb = $self->Subwidget('Listbox');
-	my $widget = $self->{WIDGET};
-
-	my $screenheight = $self->vrootheight;
-	my $height = $lb->reqheight;
-	$height = $height + $self->{FE}->reqheight if defined $self->{FE};
-	my $width = $widget->width;
-
 	my $lheight = 10;
 	if (@$list < $lheight) { $lheight = @$list }
 	$lb->configure(-height => $lheight);
-
-	my $x = $widget->rootx;
-	my $origy = $widget->rooty;
-	my $y;
-
-	if ($origy + $height + $widget->height > $screenheight) {
-		$self->{POPDIRECTION} = 'up';
-		$y = $origy - $height;
-	} else {
-		$self->{POPDIRECTION} = 'down';
-		$y = $origy + $widget->height;
-	}
-	$self->geometry(sprintf('%dx%d+%d+%d', $width, $height, $x, $y));
+	my $height = $lb->reqheight;
+	$height = $height + $self->{FE}->reqheight if defined $self->{FE};
+	return $height
 }
 
 =item B<filter>I<($filter)>
@@ -158,11 +126,15 @@ sub filter {
 	my ($self, $filter) = @_;
 	my $values = $self->{VALUES};
 	my @new = ();
+	my $len = length($filter);
 	for (@$values) {
 		push @new, $_ if $_ =~ /^$filter/i;
 	}
+	my $size = @new;
 	my $list = $self->{LIST};
-	@$list = @new;
+	#this is a hack. doing it the crude way somehow gives crashes
+	while (@$list) { pop @$list }
+	push @$list, @new;
 }
 
 sub MotionSelect {
@@ -172,12 +144,6 @@ sub MotionSelect {
 	$list->selectionSet('@' . "$x,$y");
 }
 
-=item B<popDown>
-
-Hides the PopList.
-
-=cut
-
 sub popDown {
 	my $self = shift;
 	return unless $self->ismapped;
@@ -187,35 +153,9 @@ sub popDown {
 		$e->destroy;
 		$self->{FE} = undef;
 	}
-	$self->withdraw;
-	$self->parent->grabRelease;
-	if (ref $self->{'_BE_grabinfo'} eq 'CODE') {
-		$self->{'_BE_grabinfo'}->();
-		delete $self->{'_BE_grabinfo'};
-	}
+	$self->SUPER::popDown;
 }
 
-=item B<popFlip>
-
-Hides the PopList if it shown. Shows the PopList if it is hidden.
-
-=cut
-
-sub popFlip {
-	my $self = shift;
-	if ($self->ismapped) {
-		$self->popDown
-	} else {
-		$self->popUp
-	}
-}
-
-=item B<popUp>
-
-Shows the PopList.
-
-=cut
-
 sub popUp {
 	my $self = shift;
 
@@ -223,11 +163,10 @@ sub popUp {
 
 	my $values = $self->{VALUES};
 	my $list = $self->{LIST};
-	@$list = @$values;
+	#this is a hack. doing it the crude way somehow gives crashes
+	while (@$list) { pop @$list }
+	push @$list, @$values;
 	
-	my $lb = $self->Subwidget('Listbox');
-	my $widget = $self->{WIDGET};
-
 	my $e;
 	if ($self->cget('-filter')) {
 		my $var = 'Filter';
@@ -236,36 +175,27 @@ sub popUp {
 		);
 		$e->bind('<FocusIn>', sub { $var = '' });
 		$e->bind('<Escape>', [$self, 'popDown']);
-		Tie::Watch->new(
-			-variable => \$var,
-			-store => sub {
-				my ($watch, $value) = @_;
-				$watch->Store($value);
-				$self->filter($value);
-			},
-		);
+		$e->bind('<Key>', sub { $self->filter($e->get) });
 		$self->{FE} = $e;
 	}
 
-	$self->ConfigureSizeAndPos;
+	$self->SUPER::popUp;
+
+	my $lb = $self->Subwidget('Listbox');
+	$lb->selectionClear(0, 'end');
+	$lb->selectionSet('@0,0');
+	$lb->focus;
 
 	my @filterpack = ();
-	if ($self->{POPDIRECTION} eq 'up') {
+	my $direction = $self->popDirection;
+	if ($direction eq 'up') {
 		@filterpack = (-after => $lb);
-	} elsif ($self->{POPDIRECTION} eq 'down') {
+	} elsif ($direction eq 'down') {
 		@filterpack = (-before => $lb);
 	}
 	$e->pack(@filterpack,
 		-fill => 'x'
 	) if defined $e;
-
-	$lb->selectionClear(0, 'end');
-	$lb->selectionSet('@0,0');
-	$self->deiconify;
-	$self->raise;
-	$lb->focus;
-	$self->{'_BE_grabinfo'} = $self->grabSave;
-	$self->parent->grabGlobal;
 }
 
 sub Select {
@@ -283,6 +213,7 @@ sub values {
 	$self->{VALUES} = $new if defined $new;
 	return $self->{VALUES}
 }
+
 =back
 
 =head1 LICENSE
@@ -291,13 +222,7 @@ Same as Perl.
 
 =head1 AUTHOR
 
-=over 4
-
-=item Hans Jeuken (hanje at cpan dot org)
-
-=back
-
-=cut
+Hans Jeuken (hanje at cpan dot org)
 
 =head1 BUGS
 
@@ -308,7 +233,11 @@ Unknown. If you find any, please contact the author.
 =over 4
 
 =item L<Tk>
+
+=item L<Tk::Poplevel>
+
 =item L<Tk::Toplevel>
+
 =item L<Tk::Listbox>
 
 =back
